@@ -110,7 +110,7 @@ void Core::onBackendReady()
                 this, SLOT(onDeviceRemoved(QString)));
 
         // Force the addition of already existent batteries
-        foreach (const Device &device, Device::listFromType(DeviceInterface::Battery, QString())) {
+        Q_FOREACH (const Device &device, Device::listFromType(DeviceInterface::Battery, QString())) {
             onDeviceAdded(device.udi());
         }
     }
@@ -162,7 +162,7 @@ void Core::onBackendReady()
     }
 
     // All systems up Houston, let's go!
-    emit coreReady();
+    Q_EMIT coreReady();
     refreshStatus();
 }
 
@@ -192,7 +192,7 @@ void Core::reparseConfiguration()
     m_profilesConfig->reparseConfiguration();
 
     // Config reloaded
-    emit configurationReloaded();
+    Q_EMIT configurationReloaded();
 }
 
 QString Core::currentProfile() const
@@ -239,7 +239,7 @@ void Core::loadProfile(bool force)
         profileId = activity;
     } else {
         // It doesn't, let's load the current state's profile
-        if (m_loadedBatteriesUdi.isEmpty()) {
+        if (m_batteriesPercent.isEmpty()) {
             qCDebug(POWERDEVIL) << "No batteries found, loading AC";
             profileId = "AC";
         } else if (activityConfig.readEntry("mode", "None") == "ActLike") {
@@ -318,7 +318,7 @@ void Core::loadProfile(bool force)
         }
 
         // Cool, now let's load the needed actions
-        foreach (const QString &actionName, config.groupList()) {
+        Q_FOREACH (const QString &actionName, config.groupList()) {
             Action *action = ActionPool::instance()->loadAction(actionName, config.group(actionName), this);
             if (action) {
                 action->onProfileLoad();
@@ -333,7 +333,7 @@ void Core::loadProfile(bool force)
 
         // We are now on a different profile
         m_currentProfile = profileId;
-        emit profileChanged(m_currentProfile);
+        Q_EMIT profileChanged(m_currentProfile);
     }
 
     // Now... any special behaviors we'd like to consider?
@@ -373,84 +373,79 @@ void Core::loadProfile(bool force)
 
     // If the lid is closed, retrigger the lid close signal
     if (m_backend->isLidClosed()) {
-        emit m_backend->buttonPressed(PowerDevil::BackendInterface::LidClose);
+        Q_EMIT m_backend->buttonPressed(PowerDevil::BackendInterface::LidClose);
     }
 }
 
-void Core::onDeviceAdded(const QString& udi)
+void Core::onDeviceAdded(const QString &udi)
 {
-    if (m_loadedBatteriesUdi.contains(udi)) {
+    if (m_batteriesPercent.contains(udi) || m_peripheralBatteriesPercent.contains(udi)) {
         // We already know about this device
         return;
     }
 
     using namespace Solid;
     Device device(udi);
-    Battery *b = qobject_cast<Battery*>(device.asDeviceInterface(DeviceInterface::Battery));
+    Battery *b = qobject_cast<Battery *>(device.asDeviceInterface(DeviceInterface::Battery));
 
     if (!b) {
-        // Not interesting to us
         return;
     }
 
-    if (b->type() != Solid::Battery::PrimaryBattery && b->type() != Solid::Battery::UpsBattery) {
-        // Not interesting to us
-        return;
-    }
-
-    if (!b->isPowerSupply()) {
-        // TODO: At some later point it would be really nice to handle those batteries too
-        // eg, show "your mouse is running low", but in the mean time, we don't care about those
-        return;
-    }
-
-    if (!connect(b, SIGNAL(chargePercentChanged(int,QString)),
-                 this, SLOT(onBatteryChargePercentChanged(int,QString))) ||
-        !connect(b, SIGNAL(chargeStateChanged(int,QString)),
-                 this, SLOT(onBatteryChargeStateChanged(int,QString)))) {
+    if (!connect(b, &Battery::chargePercentChanged, this, &Core::onBatteryChargePercentChanged) ||
+        !connect(b, &Battery::chargeStateChanged, this, &Core::onBatteryChargeStateChanged)) {
         emitNotification("powerdevilerror", i18n("Could not connect to battery interface.\n"
                          "Please check your system configuration"));
     }
 
-    qCDebug(POWERDEVIL) << "A new battery was detected";
+    qCDebug(POWERDEVIL) << "Battery with UDI" << udi << "was detected";
 
-    m_batteriesPercent[udi] = b->chargePercent();
-    m_batteriesCharged[udi] = (b->chargeState() == Solid::Battery::FullyCharged);
-    m_loadedBatteriesUdi.append(udi);
+    if (b->isPowerSupply()) {
+        m_batteriesPercent[udi] = b->chargePercent();
+        m_batteriesCharged[udi] = (b->chargeState() == Solid::Battery::FullyCharged);
+    } else { // non-power supply batteries are treated separately
+        m_peripheralBatteriesPercent[udi] = b->chargePercent();
 
-    const int chargePercent = currentChargePercent();
+        // notify the user about the empty mouse/keyboard when plugging it in; don't when
+        // notifications aren't ready yet so we avoid showing them ontop of ksplash;
+        // also we'll notify about all devices when notifications are ready anyway
+        if (m_notificationsReady) {
+            emitBatteryChargePercentNotification(b->chargePercent(), 1000 /* so current is always lower than previous */, udi);
+        }
+    }
 
     // If a new battery has been added, let's clear some pending suspend actions if the new global batteries percentage is
     // higher than the battery critical level. (See bug 329537)
-    if (m_criticalBatteryTimer->isActive() && chargePercent > PowerDevilSettings::batteryCriticalLevel()) {
+    if (m_criticalBatteryTimer->isActive() && currentChargePercent() > PowerDevilSettings::batteryCriticalLevel()) {
         m_criticalBatteryTimer->stop();
-        emitRichNotification("criticalbattery",
-                             i18n("Additional Battery Added"),
-                             i18n("All pending suspend actions have been canceled."));
+        if (m_criticalBatteryNotification) {
+            m_criticalBatteryNotification->close();
+        }
+        emitRichNotification("pluggedin",
+                             i18n("Extra Battery Added"),
+                             i18n("All pending suspend actions have been canceled.")); // FIXME This wording is too technical
     }
 }
 
-void Core::onDeviceRemoved(const QString& udi)
+void Core::onDeviceRemoved(const QString &udi)
 {
-    if (!m_loadedBatteriesUdi.contains(udi)) {
+    if (!m_batteriesPercent.contains(udi) && !m_peripheralBatteriesPercent.contains(udi)) {
         // We don't know about this device
         return;
     }
 
     using namespace Solid;
     Device device(udi);
-    Battery *b = qobject_cast<Battery*>(device.asDeviceInterface(DeviceInterface::Battery));
+    Battery *b = qobject_cast<Battery *>(device.asDeviceInterface(DeviceInterface::Battery));
 
-    disconnect(b, SIGNAL(chargePercentChanged(int,QString)),
-               this, SLOT(onBatteryChargePercentChanged(int,QString)));
-    disconnect(b, SIGNAL(chargeStateChanged(int,QString)),
-               this, SLOT(onBatteryChargeStateChanged(int,QString)));
+    disconnect(b, &Battery::chargePercentChanged, this, &Core::onBatteryChargePercentChanged);
+    disconnect(b, &Battery::chargeStateChanged, this, &Core::onBatteryChargeStateChanged);
 
-    qCDebug(POWERDEVIL) << "An existing battery has been removed";
+    qCDebug(POWERDEVIL) << "Battery with UDI" << udi << "has been removed";
 
     m_batteriesPercent.remove(udi);
+    m_peripheralBatteriesPercent.remove(udi);
     m_batteriesCharged.remove(udi);
-    m_loadedBatteriesUdi.removeOne(udi);
 }
 
 void Core::emitNotification(const QString &evid, const QString &message, const QString &iconname)
@@ -464,14 +459,74 @@ void Core::emitNotification(const QString &evid, const QString &message, const Q
     }
 }
 
+void Core::emitNotification(const QString &eventId, const QString &title, const QString &message, const QString &iconName)
+{
+    KNotification::event(eventId, title, message, iconName, 0, KNotification::CloseOnTimeout, "powerdevil");
+}
+
 void Core::emitRichNotification(const QString &evid, const QString &title, const QString &message)
 {
     KNotification::event(evid, title, message, QPixmap(),
                          0, KNotification::CloseOnTimeout, "powerdevil");
 }
 
-bool Core::emitBatteryChargePercentNotification(int currentPercent, int previousPercent)
+bool Core::emitBatteryChargePercentNotification(int currentPercent, int previousPercent, const QString &udi)
 {
+    using namespace Solid;
+    Device device(udi);
+    Battery *b = qobject_cast<Battery *>(device.asDeviceInterface(DeviceInterface::Battery));
+
+    if (b && !b->isPowerSupply()) {
+        // if you leave the device out of reach or it has not been initialized yet
+        // it won't be "there" and report 0%, don't show anything in this case
+        if (!b->isPresent()) {
+            return false;
+        }
+
+        if (currentPercent <= PowerDevilSettings::peripheralBatteryLowLevel() &&
+            previousPercent > PowerDevilSettings::peripheralBatteryLowLevel()) {
+
+            QString name = device.product();
+            if (!device.vendor().isEmpty()) {
+                name = i18nc("%1 is vendor name, %2 is product name", "%1 %2", device.vendor(), device.product());
+            }
+
+            QString title;
+            QString msg;
+            QString icon;
+
+            switch(b->type()) {
+            case Battery::MouseBattery:
+                title = i18n("Mouse Battery Low (%1% Remaining)", currentPercent);
+                msg = i18nc("Placeholder is device name",
+                            "The battery in your mouse (\"%1\") is low, and the device may turn itself off at any time. "
+                            "Please replace or charge the battery as soon as possible.", name);
+                icon = QStringLiteral("input-mouse");
+                break;
+            case Battery::KeyboardBattery:
+                title = i18n("Keyboard Battery Low (%1% Remaining)", currentPercent);
+                msg = i18nc("Placeholder is device name",
+                            "The battery in your keyboard (\"%1\") is low, and the device may turn itself off at any time. "
+                            "Please replace or charge the battery as soon as possible.", name);
+                icon = QStringLiteral("input-keyboard");
+                break;
+            default:
+                title = i18nc("The battery in an external device", "Device Battery Low (%1% Remaining)", currentPercent);
+                msg = i18nc("Placeholder is device name",
+                            "The battery in a connected device (\"%1\") is low, and the device may turn itself off at any time. "
+                            "Please replace or charge the battery as soon as possible.", name);
+                icon = QStringLiteral("battery-caution");
+                break;
+            }
+
+            emitNotification("lowperipheralbattery", title, msg, icon);
+
+            return true;
+        }
+
+        return false;
+    }
+
     if (m_backend->acAdapterState() == BackendInterface::Plugged) {
         return false;
     }
@@ -539,7 +594,10 @@ void Core::onAcAdapterStateChanged(PowerDevil::BackendInterface::AcAdapterState
         // If the AC Adaptor has been plugged in, let's clear some pending suspend actions
         if (m_criticalBatteryTimer->isActive()) {
             m_criticalBatteryTimer->stop();
-            emitRichNotification("criticalbattery",
+            if (m_criticalBatteryNotification) {
+                m_criticalBatteryNotification->close();
+            }
+            emitRichNotification("pluggedin",
                              i18n("AC Adapter Plugged In"),
                              i18n("All pending suspend actions have been canceled."));
         } else {
@@ -559,6 +617,16 @@ void Core::onBackendError(const QString& error)
 
 void Core::onBatteryChargePercentChanged(int percent, const QString &udi)
 {
+    if (m_peripheralBatteriesPercent.contains(udi)) {
+        const int previousPercent = m_peripheralBatteriesPercent.value(udi);
+        m_peripheralBatteriesPercent[udi] = percent;
+
+        if (percent < previousPercent) {
+            emitBatteryChargePercentNotification(percent, previousPercent, udi);
+        }
+        return;
+    }
+
     // Compute the previous and current global percentage
     const int previousPercent = currentChargePercent();
     const int currentPercent = previousPercent - (m_batteriesPercent[udi] - percent);
@@ -567,7 +635,7 @@ void Core::onBatteryChargePercentChanged(int percent, const QString &udi)
     m_batteriesPercent[udi] = percent;
 
     if (currentPercent < previousPercent) {
-        if (emitBatteryChargePercentNotification(currentPercent, previousPercent)) {
+        if (emitBatteryChargePercentNotification(currentPercent, previousPercent, udi)) {
             // Only refresh status if a notification has actually been emitted
             loadProfile();
         }
@@ -576,6 +644,10 @@ void Core::onBatteryChargePercentChanged(int percent, const QString &udi)
 
 void Core::onBatteryChargeStateChanged(int state, const QString &udi)
 {
+    if (!m_batteriesCharged.contains(udi)) {
+        return;
+    }
+
     bool previousCharged = true;
     for (auto i = m_batteriesCharged.constBegin(); i != m_batteriesCharged.constEnd(); ++i) {
         if (!i.value()) {
@@ -625,7 +697,7 @@ void Core::onCriticalBatteryTimerExpired()
 
 void Core::onBatteryRemainingTimeChanged(qulonglong time)
 {
-    emit batteryRemainingTimeChanged(time);
+    Q_EMIT batteryRemainingTimeChanged(time);
 }
 
 void Core::onKIdleTimeoutReached(int identifier, int msec)
@@ -651,7 +723,7 @@ void Core::onKIdleTimeoutReached(int identifier, int msec)
 
 void Core::onLidClosedChanged(bool closed)
 {
-    emit lidClosedChanged(closed);
+    Q_EMIT lidClosedChanged(closed);
 }
 
 void Core::registerActionTimeout(Action* action, int timeout)
@@ -670,7 +742,7 @@ void Core::unregisterActionTimeouts(Action* action)
     // Clear all timeouts from the action
     QList< int > timeoutsToClean = m_registeredActionTimeouts[action];
 
-    foreach (int id, timeoutsToClean) {
+    Q_FOREACH (int id, timeoutsToClean) {
         KIdleTime::instance()->removeIdleTimeout(id);
     }
 
@@ -706,19 +778,31 @@ void Core::onServiceRegistered(const QString &service)
 {
     Q_UNUSED(service);
 
-    static bool notificationsReady = false;
-    if (notificationsReady) {
+    if (m_notificationsReady) {
         return;
     }
 
+    bool needsRefresh = false;
+
     // show warning about low batteries right on session startup, force it to show
     // by making sure the "old" percentage (that magic number) is always higher than the current one
     if (emitBatteryChargePercentNotification(currentChargePercent(), 1000)) {
-        // need to refresh status to prevent the notification from showing again when charge percentage changes
+        needsRefresh = true;
+    }
+
+    // now also emit notifications for all peripheral batteries
+    for (auto it = m_peripheralBatteriesPercent.constBegin(), end = m_peripheralBatteriesPercent.constEnd(); it != end; ++it) {
+        if (emitBatteryChargePercentNotification(it.value() /*currentPercent*/, 1000, it.key() /*udi*/)) {
+            needsRefresh = true;
+        }
+    }
+
+    // need to refresh status to prevent the notification from showing again when charge percentage changes
+    if (needsRefresh) {
         refreshStatus();
     }
 
-    notificationsReady = true;
+    m_notificationsReady = true;
 
     if (m_notificationsWatcher) {
         delete m_notificationsWatcher;
